@@ -19,8 +19,8 @@
  * @file globus_ftp_client_restart_plugin.c GridFTP Restart Plugin Implementation
  *
  * $RCSfile: globus_ftp_client_restart_plugin.c,v $
- * $Revision: 1.24 $
- * $Date: 2009/12/17 20:27:20 $
+ * $Revision: 1.24.4.1 $
+ * $Date: 2011/02/08 20:35:08 $
  */
 #endif
 
@@ -947,6 +947,14 @@ l_ticker_cb(
                 default:
                     globus_assert(0 && "should never happen--memory corruption");
             }
+            if(result == GLOBUS_SUCCESS)
+            {
+            }
+            else if(d->max_retries >= 0)
+            {
+                d->max_retries++;
+            }
+                
         }
     }
     d->ticker++;
